from datetime import datetime
from typing import Union
import pandas as pd

from .FiinIndicator import _FiinIndicator
from .Fetch_Trading_Data import Fetch_Trading_Data
from .Trading_Data_Stream import Trading_Data_Stream
from .DateCorr import FindDateCorrelation


class FiinSession:
    """
    This class is used to create a session to access FiinQuant.
    """
    def __init__(self, username: str, password: str):...

    def login(self) -> FiinSession: ...
        
    def FiinIndicator(self) -> _FiinIndicator: 
        """
        FiinIndicator class is used to calculate technical indicators for stock market data.
        """
    def Trading_Data_Stream(self, tickers: list, callback: callable) -> Trading_Data_Stream: 
        """
        Using this class to stream real-time stock market matching data. Using start() to start streaming and stop() to stop streaming.
        The variable _stop is used to check if the streaming is stopped or not."""
    def Fetch_Trading_Data(self, realtime: bool, tickers: list, fields: list, adjusted: bool, 
                           period: Union[int, None] = None, by: str='1M',
                           from_date: Union[str, datetime, None] = None,
                           to_date: Union[str, datetime, None] = None,
                           callback: callable = None,
                           wait_for_full_timeFrame: bool = False) -> Fetch_Trading_Data: 

        """summary:
        
        This function is used to fetch trading data from the server. The data can be fetched in real-time or historical data.
        callback and wait_for_full_timeFrame only required when realtime = True. Two method start() and stop() are used to start and stop (Only needed when realtime = True).
        
        Args:
            realtime (bool): Real-time data or not.
            tickers (list): List of tickers.
            fields (list): List of fields. Including: 'O', 'H', 'L', 'C', 'V', 'B', 'S', 'FB', 'FS', 'FN'.
            adjusted (bool, optional): Adjusted price or unadjusted price. Default to True (Adjusted).
            period (Union[int, None], optional): Number of most recent candles. Default to None.
            by (str, optional): Unit of time. Default to '1M', '5M', '15M', '30M', '1H', '3H', '4H', '1D'.
            from_date (Union[str, datetime, None], optional): The furthest time of data. Default to None.
            to_date (Union[str, datetime, None], optional): Latest time of data. Default to None.
            callback (callable, optional): User-defined callback function for manipulating data. Default to None.
            wait_for_full_timeFrame (bool, optional): Wait until the candle is over to get data or not. Default to False.
        """
    def FindDateCorrelation(self) -> FindDateCorrelation: 
        """
        Using this class to find the correlation between today data and the past data.
        """
        
class FindDateCorrelation(object):
    """
    Using this class to find the correlation between today data and the past data.
    """
    def __init__(self, access_token: str) -> None: ...
    
    def intraday_Prediction(self, Tickers: Union[str, list], Timeframe: str, 
                            t1: str = None, t2: str = None, method: str = "pearson correlation",
                            year: int = 1, field=['C'], adjusted=True) -> None:
        """
        Plot the normalized price of a stock over time, comparing the current day to the past days.
        method (str, optional): Distance measurement. Default to "pearson correlation".
        For simple, use '1' for euclidean distance, '2' for dynamic time warping, 
        '3' for pearson correlation, '4' for cosine similarity.
        """
        self.field: list [str]
        self.adjusted: bool

class RealTimeData:
    """
    RealTimeData is a class that represents a real-time data of a stock. This class is 
    a structure that contains all the information of a stock at a specific time. You can use 
    to_dataFrame() method to convert this class to a pandas DataFrame.
    """
    def __init__(self, data: pd.DataFrame) -> None:
        self.__private_attribute = data
        self.Ticker: str
        self.TotalMatchVolume: int
        self.MarketStatus: str
        self.TradingDate: str
        self.ComGroupCode: str
        self.ReferencePrice: float
        self.Open: float
        self.Close: float
        self.High: float
        self.Low: float
        self.Change: float
        self.ChangePercent: float
        self.MatchVolume: int
        self.MatchValue: float
        self.TotalMatchValue: float
        self.TotalBuyTradeVolume: int
        self.TotalSellTradeVolume: int
        self.TotalDealVolume: int
        self.TotalDealValue: float
        self.ForeignBuyVolumeTotal: int
        self.ForeignBuyValueTotal: float
        self.ForeignSellVolumeTotal: int
        self.ForeignSellValueTotal: float
        
    def to_dataFrame(self) -> pd.DataFrame: ...

class BarDataUpdate:
    """
    This class is used to store both the real-time data and the historical data of a stock. You can use to_dataFrame() method to convert this data to a pandas.
    
    """
    def __init__(self, data) -> None:
        self.__private_attribute: pd.DataFrame
        self.Timestamp: Union[str, datetime]
        self.Open: float
        self.High: float
        self.Low: float
        self.Close: float
        self.Volume: int
        self.Ticker: str 
        self.BU: int
        self.SD: int
        self.FB: float
        self.FS: float
        self.FN: float

    def to_dataFrame(self) -> pd.DataFrame: ...

